package cn.iocoder.yudao.module.shipment.service.boxPkgOrder;

import cn.hutool.core.collection.CollectionUtil;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQueryWrapperX;
import cn.iocoder.yudao.framework.mybatis.core.service.AbstractService;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.module.order.dal.dataobject.orderItem.OrderItemDO;
import cn.iocoder.yudao.module.order.service.order.OrderItemService;
import cn.iocoder.yudao.module.shipment.convert.BoxPkgOrderConvert;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxDO;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxMergePkgDO;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxPkgOrderDO;
import cn.iocoder.yudao.module.shipment.dal.dataobject.boxOrderCheckInfo.BoxOrderCheckInfoDO;
import cn.iocoder.yudao.module.shipment.dal.mysql.BoxPkgOrderMapper;
import cn.iocoder.yudao.module.shipment.enums.AirShipmentStatusEnum;
import cn.iocoder.yudao.module.shipment.enums.MergePkgStatusEnum;
import cn.iocoder.yudao.module.shipment.enums.OrderShipmentLog;
import cn.iocoder.yudao.module.shipment.service.box.BoxService;
import cn.iocoder.yudao.module.shipment.service.boxMergePkg.BoxMergePkgService;
import cn.iocoder.yudao.module.shipment.service.boxOrderCheckInfo.BoxOrderCheckInfoService;
import cn.iocoder.yudao.module.shipment.vo.boxPkgOrder.BoxPkgOrderCreateReqVO;
import cn.iocoder.yudao.module.shipment.vo.boxPkgOrder.BoxPkgOrderQueryVO;
import cn.iocoder.yudao.module.shipment.vo.boxPkgOrder.BoxPkgOrderUpdateReqVO;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.shipment.enums.ErrorCodeConstants.BOX_PKG_ORDER_NOT_EXISTS;
import static cn.iocoder.yudao.module.shipment.enums.ErrorCodeConstants.MERGE_PKG_IS_FINISH;

/**
 * 合包订单 Service 实现类
 *
 * @author 江楠
 */
@Service
@Validated
public class BoxPkgOrderServiceImpl extends AbstractService<BoxPkgOrderMapper, BoxPkgOrderDO> implements BoxPkgOrderService {

    @Resource
    private BoxPkgOrderMapper boxPkgOrderMapper;
    
    @Resource
    private OrderItemService orderItemService;

    @Resource
    private BoxMergePkgService boxMergePkgService;

    @Resource
    @Lazy
    private BoxService boxService;

    @Resource
    @Lazy
    private BoxOrderCheckInfoService boxOrderCheckInfoService;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public String createBoxPkgOrder(BoxPkgOrderCreateReqVO createReqVO) {
        create(createReqVO);
        // 返回
        return "success";
    }


    private void create(BoxPkgOrderCreateReqVO createReqVO) {
        checkMergePkgStatus(createReqVO.getPkgId());
        List<OrderItemDO> orderItemDOS = orderItemService.selectList(OrderItemDO::getOrderId, createReqVO.getOrderId());
        orderItemDOS.stream().forEach(item -> {
            // 插入
            BoxPkgOrderDO boxPkgOrder = BoxPkgOrderConvert.INSTANCE.convert(createReqVO);
            boxPkgOrder.setOrderItemId(item.getOrderItemId());
            boxPkgOrderMapper.insert(boxPkgOrder);
        });
        Long aLong = boxPkgOrderMapper.selectCount(BoxPkgOrderDO::getPkgId, createReqVO.getPkgId());
        if (aLong.longValue() > 0L) {
            boxMergePkgService.update(new LambdaUpdateWrapper<BoxMergePkgDO>().set(BoxMergePkgDO::getStatus,2).eq(BoxMergePkgDO::getId,createReqVO.getPkgId()));
        }
        boxService.addOrderLog(Collections.singletonList(createReqVO.getOrderId()), OrderShipmentLog.MERGE_PKG_ADD, "");
    }

    @Override
    public void updateBoxPkgOrder(BoxPkgOrderUpdateReqVO updateReqVO) {
        // 校验存在
        this.validateBoxPkgOrderExists(updateReqVO.getId());
        // 更新
        BoxPkgOrderDO updateObj = BoxPkgOrderConvert.INSTANCE.convert(updateReqVO);
        boxPkgOrderMapper.updateById(updateObj);
    }

    @Override
    public void deleteBoxPkgOrder(Long id) {
        // 校验存在
        this.validateBoxPkgOrderExists(id);
        // 删除
        boxPkgOrderMapper.deleteById(id);
    }

    private void validateBoxPkgOrderExists(Long id) {
        if (boxPkgOrderMapper.selectById(id) == null) {
            throw exception(BOX_PKG_ORDER_NOT_EXISTS);
        }
    }

    @Override
    public BoxPkgOrderDO getBoxPkgOrder(Long id) {
        return boxPkgOrderMapper.selectById(id);
    }

    @Override
    public List<BoxPkgOrderDO> getBoxPkgOrderList(Collection<Long> ids) {
        return boxPkgOrderMapper.selectBatchIds(ids);
    }

    @Override
    public PageResult<BoxPkgOrderDO> getBoxPkgOrderPage(BoxPkgOrderQueryVO query, PageVO page) {
        return boxPkgOrderMapper.selectPage(page, query);
    }

    @Override
    public List<BoxPkgOrderDO> getBoxPkgOrderList(BoxPkgOrderQueryVO query) {
        return boxPkgOrderMapper.selectList(query);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public String createBatch(List<BoxPkgOrderCreateReqVO> createReqVO) {
        if (CollectionUtil.isNotEmpty(createReqVO)) {
            createReqVO.stream().forEach(item ->{
                create(item);
            });
        }
        return "success";
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void deleteRelate(Long pkgId, Long orderId) {
        checkMergePkgStatus(pkgId);
        boxPkgOrderMapper.delete(new LambdaQueryWrapperX<BoxPkgOrderDO>().eq(BoxPkgOrderDO::getPkgId,pkgId).eq(BoxPkgOrderDO::getOrderId,orderId));
        boxOrderCheckInfoService.remove(new LambdaQueryWrapperX<BoxOrderCheckInfoDO>()
                .eq(BoxOrderCheckInfoDO::getOrderId,orderId)
                .eq(BoxOrderCheckInfoDO::getType,1)
        );
        Long aLong = boxPkgOrderMapper.selectCount(BoxPkgOrderDO::getPkgId, pkgId);
        if (aLong.longValue() <= 0L) {
            boxMergePkgService.update(new LambdaUpdateWrapper<BoxMergePkgDO>().set(BoxMergePkgDO::getStatus,1).eq(BoxMergePkgDO::getId,pkgId));
            boxService.addOrderLog(Collections.singletonList(orderId), OrderShipmentLog.MERGE_PKG_REMOVE, "");
        }
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void deleteBatchRelate(List<BoxPkgOrderQueryVO> list) {
        if (CollectionUtil.isNotEmpty(list)) {
            checkMergePkgStatus(list.get(0).getPkgId());
            List<Long> orderIdList = list.stream()
                    .map(BoxPkgOrderQueryVO::getOrderId)
                    .collect(Collectors.toList());
            boxService.addOrderLog(orderIdList, OrderShipmentLog.MERGE_PKG_REMOVE, "");
            list.stream().forEach(item -> {
                boxPkgOrderMapper.delete(new LambdaQueryWrapperX<BoxPkgOrderDO>().eq(BoxPkgOrderDO::getPkgId,item.getPkgId()).eq(BoxPkgOrderDO::getOrderId,item.getOrderId()));
                boxOrderCheckInfoService.remove(new LambdaQueryWrapperX<BoxOrderCheckInfoDO>()
                        .eq(BoxOrderCheckInfoDO::getOrderId,item.getOrderId())
                        .eq(BoxOrderCheckInfoDO::getType,1)
                );
            });
            Long aLong = boxPkgOrderMapper.selectCount(BoxPkgOrderDO::getPkgId, list.get(0).getPkgId());
            if (aLong.longValue() <= 0L) {
                boxMergePkgService.update(new LambdaUpdateWrapper<BoxMergePkgDO>().set(BoxMergePkgDO::getStatus,1).eq(BoxMergePkgDO::getId,list.get(0).getPkgId()));
            }
        }
    }

    @Override
    public void deleteRelateByOrderIds(Long shipmentId, List<Long> orderIds) {
        List<BoxMergePkgDO> boxMergePkgDOS = boxMergePkgService.selectList(BoxMergePkgDO::getShipmentId, shipmentId);
        List<Long> pkgIds = boxMergePkgDOS.stream().map(BoxMergePkgDO::getId).collect(Collectors.toList());
        if (CollectionUtil.isNotEmpty(pkgIds) && CollectionUtil.isNotEmpty(orderIds)) {
            boxPkgOrderMapper.delete(new LambdaQueryWrapperX<BoxPkgOrderDO>().in(BoxPkgOrderDO::getPkgId,pkgIds).in(BoxPkgOrderDO::getOrderId,orderIds));
        }
    }

    @Override
    public Map<String, Integer> getMergeOrderQualityByPkgIds(List<Long> pkgIds) {
        List<BoxMergePkgDO> mergeOrderQualityByPkgIds = boxPkgOrderMapper.getMergeOrderQualityByPkgIds(pkgIds);
        Map<String,Integer> map = new HashMap<>();
        mergeOrderQualityByPkgIds.stream().forEach(item -> {
            map.put(String.valueOf(item.getId()),item.getSumNum().intValue());
        });
        return map;
    }

    private void checkMergePkgStatus(Long pkgId) {
        BoxMergePkgDO boxMergePkgDO = boxMergePkgService.selectOne(BoxMergePkgDO::getId, pkgId);
        BoxDO box = boxService.getBox(boxMergePkgDO.getShipmentId());
        if (box.getAirShipmentStatus() != AirShipmentStatusEnum.UNSHIPMENT.getStatus()) {
            throw exception(MERGE_PKG_IS_FINISH);
        }
    }
}
