package cn.iocoder.yudao.module.shipment.controller.admin;

import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.module.shipment.convert.BoxPkgOrderConvert;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxPkgOrderDO;
import cn.iocoder.yudao.module.shipment.service.boxPkgOrder.BoxPkgOrderService;
import cn.iocoder.yudao.module.shipment.vo.boxPkgOrder.*;
import org.springframework.web.bind.annotation.*;
import javax.annotation.Resource;
import org.springframework.validation.annotation.Validated;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.annotations.*;
import javax.validation.constraints.*;
import javax.validation.*;
import javax.servlet.http.*;
import java.util.*;
import java.io.IOException;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;
import cn.iocoder.yudao.framework.excel.util.ExcelUtils;
import cn.iocoder.yudao.framework.operatelog.core.annotations.OperateLog;
import static cn.iocoder.yudao.framework.operatelog.core.enums.OperateTypeEnum.*;

@Validated
@RestController
@Api(tags = "管理后台 - 合包订单")
@RequestMapping("/ecw/box-pkg-order")
public class BoxPkgOrderController {

    @Resource
    private BoxPkgOrderService boxPkgOrderService;

    @PostMapping("/create")
    @ApiOperation("创建合包订单")
    //@PreAuthorize("@ss.hasPermission('ecw:box-pkg-order:create')")
    public CommonResult<String> createBoxPkgOrder(@Valid @RequestBody BoxPkgOrderCreateReqVO createReqVO) {
        return success(boxPkgOrderService.createBoxPkgOrder(createReqVO));
    }

    @PostMapping("/createBatch")
    @ApiOperation("批量创建合包订单")
    //@PreAuthorize("@ss.hasPermission('ecw:box-pkg-order:create')")
    public CommonResult<String> createBatch(@Valid @RequestBody List<BoxPkgOrderCreateReqVO> createReqVO) {
        return success(boxPkgOrderService.createBatch(createReqVO));
    }

    @PutMapping("/update")
    @ApiOperation("更新合包订单")
    //@PreAuthorize("@ss.hasPermission('ecw:box-pkg-order:update')")
    public CommonResult<Boolean> updateBoxPkgOrder(@Valid @RequestBody BoxPkgOrderUpdateReqVO updateReqVO) {
        boxPkgOrderService.updateBoxPkgOrder(updateReqVO);
        return success(true);
    }

    @DeleteMapping("/delete")
    @ApiOperation("删除合包订单")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    //@PreAuthorize("@ss.hasPermission('ecw:box-pkg-order:delete')")
    public CommonResult<Boolean> deleteBoxPkgOrder(@RequestParam("id") Long id) {
        boxPkgOrderService.deleteBoxPkgOrder(id);
        return success(true);
    }

    @DeleteMapping("/deleteRelate")
    @ApiOperation("移出合包订单")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "pkgId", value = "合包箱号ID", required = true, example = "1024", dataTypeClass = Long.class),
            @ApiImplicitParam(name = "orderId", value = "订单ID", required = true, example = "1024", dataTypeClass = Long.class)
    })
    //@PreAuthorize("@ss.hasPermission('ecw:box-pkg-order:delete')")
    public CommonResult<Boolean> deleteRelate(@RequestParam("pkgId") Long pkgId,@RequestParam("orderId") Long orderId) {
        boxPkgOrderService.deleteRelate(pkgId,orderId);
        return success(true);
    }

    @PostMapping("/deleteBatchRelate")
    @ApiOperation("批量移出合包订单")
    //@PreAuthorize("@ss.hasPermission('ecw:box-pkg-order:delete')")
    public CommonResult<Boolean> deleteBatchRelate(@RequestBody List<BoxPkgOrderQueryVO> list) {
        boxPkgOrderService.deleteBatchRelate(list);
        return success(true);
    }

    @GetMapping("/get")
    @ApiOperation("获得合包订单")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    //@PreAuthorize("@ss.hasPermission('ecw:box-pkg-order:query')")
    public CommonResult<BoxPkgOrderBackVO> getBoxPkgOrder(@RequestParam("id") Long id) {
        BoxPkgOrderDO boxPkgOrder = boxPkgOrderService.getBoxPkgOrder(id);
        return success(BoxPkgOrderConvert.INSTANCE.convert(boxPkgOrder));
    }

    @GetMapping("/list")
    @ApiOperation("获得合包订单列表")
    @ApiImplicitParam(name = "ids", value = "编号列表", required = true, example = "1024,2048", dataTypeClass = List.class)
    //@PreAuthorize("@ss.hasPermission('ecw:box-pkg-order:query')")
    public CommonResult<List<BoxPkgOrderBackVO>> getBoxPkgOrderList(@RequestParam("ids") Collection<Long> ids) {
        List<BoxPkgOrderDO> list = boxPkgOrderService.getBoxPkgOrderList(ids);
        return success(BoxPkgOrderConvert.INSTANCE.convertList(list));
    }

    @GetMapping("/page")
    @ApiOperation("获得合包订单分页")
    //@PreAuthorize("@ss.hasPermission('ecw:box-pkg-order:query')")
    public CommonResult<PageResult<BoxPkgOrderBackVO>> getBoxPkgOrderPage(@Valid BoxPkgOrderQueryVO query, PageVO page) {
        PageResult<BoxPkgOrderDO> pageResult = boxPkgOrderService.getBoxPkgOrderPage(query, page);
        return success(BoxPkgOrderConvert.INSTANCE.convertPage(pageResult));
    }

    @GetMapping("/export-excel")
    @ApiOperation("导出合包订单 Excel")
    //@PreAuthorize("@ss.hasPermission('ecw:box-pkg-order:export')")
    @OperateLog(type = EXPORT)
    public void exportBoxPkgOrderExcel(@Valid BoxPkgOrderQueryVO query,
              HttpServletResponse response) throws IOException {
        List<BoxPkgOrderDO> list = boxPkgOrderService.getBoxPkgOrderList(query);
        // 导出 Excel
        List<BoxPkgOrderBackVO> datas = BoxPkgOrderConvert.INSTANCE.convertList(list);
        ExcelUtils.write(response, "合包订单.xls", "数据", BoxPkgOrderBackVO.class, datas);
    }

}
